<?php

namespace WPTranslation\Controllers;

use Exception;
use WPTranslation\Api\Gumroad;
use WPTranslation\Api\OpenAI;
use WPTranslation\Controllers\Controller;
use WPTranslation\Utils\License;
use WPTranslation\Utils\Settings;
use WPTranslation\Vendors\DeepL\AuthorizationException;
use WPTranslation\Vendors\DeepL\Translator as DeepLTranslator;
use WPTranslation\Vendors\Illuminate\Support\Arr;
use WPTranslation\Vendors\Illuminate\Validation\Rule;

class SettingsController extends Controller
{
    public function index()
    {
        $this->verifyNonce();

        $this->response(Settings::get());
    }

    public function update()
    {
        $this->verifyNonce();

        $this->validate([
            "license.key" => ["nullable"],
            "service" => ["required", Rule::in(["google-translate-free", "deepl", "openai"])],
            "api.deepl.key" => ["required_if:service,deepl", "string"],
            "api.openai.key" => ["required_if:service,openai", "string"],
            "translation.post_meta.enabled" => ["boolean"],
            "translation.post_meta.blacklist" => ["array"],
            "translation.yoast.enabled" => ["boolean"],
            "translation.rank_math.enabled" => ["boolean"],
            "translation.seopress.enabled" => ["boolean"],
            "concurrent_translations" => ["integer", "min:1", "max:30"],
            "live.enabled" => ["boolean"],
            "live.languages" => ["array"],
            "live.refresh" => ["boolean"],
            "convert_links.enabled" => ["boolean"],
            "use_background_tasks" => ["boolean"],
            "set_post_in_draft" => ["boolean"],
            "concurrent_translations" => ["integer", "min:1", "max:30"],
        ], [], [
            "api.deepl.key" => "DeepL Api Key",
            "api.openai.key" => "OpenAI Api Key",
        ]);

        // validate license key
        if (!empty(Arr::get($this->request, "license.key"))) {

            $api = new Gumroad(Arr::get($this->request, "license.key"));
            try {
                $result = $api->verifyLicence();
                if (!Arr::get($result, "success", false)) {
                    $message = Arr::get($result, "message", "That license does not exist for the provided product.");
                    $this->response(["message" => "Invalid key", "errors" => ['license.key' => [$message]]], 422);
                }
            } catch (Exception $e) {
                $this->response(["message" => $e->getMessage(), "errors" => ['license.key' => [$e->getMessage()]]], 422);
            }
        } else {
            License::removeLicense();
        }

        // validate deepl api key
        if (Arr::get($this->request, "service") === 'deepl') {
            try {
                $translator = new DeepLTranslator(Arr::get($this->request, "api.deepl.key"));
                $translator->getSourceLanguages();
            } catch (AuthorizationException $e) {
                $this->response(["message" => $e->getMessage(), "errors" => ['api.deepl.key' => [$e->getMessage()]]], 422);
            }
        }

        // validate openai api key
        if (Arr::get($this->request, "service") === 'openai') {
            $api = new OpenAI(Arr::get($this->request, "api.openai.key"));
            $result = $api->listModels();
            if (!empty($result["error"])) {
                $this->response(["message" => $result["error"]["message"], "errors" => ['api.openai.key' => [$result["error"]["message"]]]], 422);
            }
        }

        Settings::update($this->request);

        Settings::save();

        $this->emptyResponse();
    }
}
