<?php

namespace WPTranslation\Controllers;

use WPTranslation\Translator;
use WPTranslation\Utils\Detector;
use WPTranslation\Vendors\Illuminate\Pagination\LengthAwarePaginator;
use WPTranslation\Vendors\Illuminate\Support\Arr;
use WP_Query;
use WPTranslation\Utils\Scheduler;
use WPTranslation\Vendors\Carbon\Carbon;

class TranslateController extends Controller
{
    private $perPage = 25;

    public function index()
    {
        $this->verifyNonce();

        $this->validate([
            "page" => ["nullable", "numeric"],
            "per_page" => ["nullable", "numeric"],
            "post_type" => ["nullable", "string"],
            "category" => ["nullable", "numeric"],
            "author" => ["nullable", "numeric"],
            "search" => ["nullable", "string"],
            "sort_order" => ["nullable", "string"],
            "sort_direction" => ["nullable", "string"],
        ]);

        if (!Detector::polylangIsActive()) {
            $this->response(["message" => "Polylang not detected.", "error" => "Polylang is missing. Activate it to use WPTranslation."], 503);
        }

        // get default language
        $language = pll_default_language();
        $languages = pll_the_languages(['raw' => 1]);

        $page = Arr::get($this->request, "page", 1);

        // get posts link to default language
        $args = [
            'posts_per_page' => Arr::get($this->request, "per_page", $this->perPage),
            'paged' => $page,
            'fields' => "ids",
            'tax_query' => [
                [
                    'taxonomy' => 'language',
                    'field' => 'term_id',
                    'terms' => $languages[$language]["id"]
                ]
            ]
        ];

        // make query
        $query = $this->makeQuery($args, $this->request);

        // create output
        $posts = [];
        foreach ($query->posts as $postId) {
            $author = get_post_field("post_author", $postId);
            $posts[] = [
                "id" => $postId,
                "post_author" => [
                    'id' => $author,
                    'name' => get_the_author_meta('display_name', $author),
                ],
                "post_date" => get_post_field("post_date", $postId),
                "post_title" => get_post_field("post_title", $postId),
                "post_status" => get_post_field("post_status", $postId),
                "categories" => wp_get_post_categories($postId, ['fields' => 'names']),
                "tags" => wp_get_post_tags($postId, ['fields' => 'names']),
                "language" => pll_get_post_language($postId),
                "translations" => pll_get_post_translations($postId),
            ];

            clean_post_cache($postId);
        }

        // pagination
        $paginator = new LengthAwarePaginator($posts, $query->found_posts, Arr::get($this->request, "per_page", $this->perPage), $page);

        $this->response($paginator->toArray());
    }

    public function getPostsToTranslate()
    {
        $this->verifyNonce();

        $this->validate([
            "filters.page" => ["nullable", "numeric"],
            "filters.per_page" => ["nullable", "numeric"],
            "filters.page" => ["nullable", "numeric"],
            "filters.post_type" => ["nullable", "string"],
            "filters.category" => ["nullable", "numeric"],
            "filters.author" => ["nullable", "numeric"],
            "filters.search" => ["nullable", "string"],
            "filters.sort_order" => ["nullable", "string"],
            "filters.sort_direction" => ["nullable", "string"],
            "selected.all" => ["boolean"],
            "selected.current" => ["boolean"],
            "selected.selected" => ["array"],
        ]);

        $this->verifyPremium();

        if (!Detector::polylangIsActive()) {
            $this->response(["message" => "Polylang not detected.", "error" => "Polylang is missing. Activate it to use WPTranslation."], 503);
        }

        // get default language
        $defaultLanguage = pll_default_language();
        $languages = pll_the_languages(['raw' => 1]);

        $page = Arr::get($this->request, "filters.page", 1);

        // get posts link to default language
        $args = [
            'lang' => $defaultLanguage,
            "fields" => "id=>parent",
            'tax_query' => [
                [
                    'taxonomy' => 'language',
                    'field' => 'term_id',
                    'terms' => $languages[$defaultLanguage]["id"]
                ]
            ]
        ];

        // selected
        if (Arr::get($this->request, "selected.all", false)) {
            $args["nopaging"] = true;
        } else if (Arr::get($this->request, "selected.current", false)) {
            $args["posts_per_page"] = Arr::get($this->request, "per_page", $this->perPage);
            $args["paged"] = $page;
        } else if (!empty(Arr::get($this->request, "selected.selected", []))) {
            $args["nopaging"] = true;
            $args["post__in"] = Arr::get($this->request, "selected.selected");
        } else {
            $this->response([]);
        }

        // make query
        $query = $this->makeQuery($args, Arr::get($this->request, 'filters', []));

        // create output
        $posts = [];
        foreach ($query->posts as $post) {
            $translations = [];

            $postTranslations = array_keys(pll_get_post_translations($post->ID));

            foreach (Arr::get($this->request, "languages", []) as $lang) {
                if ($lang === $defaultLanguage) {
                    continue;
                }

                if (!in_array($lang, $postTranslations) || Arr::get($this->request, "refresh", false)) {
                    $translations[] = $lang;
                }
            }

            $posts[] = [
                "id" => $post->ID,
                "parent_id" => $post->post_parent,
                "translations" => $translations,
            ];
        }

        if (empty($posts)) {
            $this->response([]);
        }

        // remove with empty translations
        $posts = array_filter($posts, function ($post) {
            return !empty($post["translations"]);
        });

        // key by id
        $posts = array_combine(Arr::pluck($posts, 'id'), $posts);

        if (count($posts) > 0) {
            // make tree
            $new = [];
            foreach ($posts as &$post) {
                if (isset($posts[$post['parent_id']])) {
                    $new[$post['parent_id']][] = $post;
                } else {
                    $new[0][] = $post;
                }
            }
            $tree = $this->createTree($new, $new[0]);

            $this->response($tree);
        }

        $this->response([]);
    }

    public function getPostsToRewrite()
    {
        $this->verifyNonce();

        $this->validate([
            "filters.page" => ["nullable", "numeric"],
            "filters.per_page" => ["nullable", "numeric"],
            "filters.post_type" => ["nullable", "string"],
            "filters.category" => ["nullable", "numeric"],
            "filters.author" => ["nullable", "numeric"],
            "filters.search" => ["nullable", "string"],
            "filters.sort_order" => ["nullable", "string"],
            "filters.sort_direction" => ["nullable", "string"],
            "selected.all" => ["boolean"],
            "selected.current" => ["boolean"],
            "selected.selected" => ["array"],
        ]);

        $this->verifyPremium();

        if (!Detector::polylangIsActive()) {
            $this->response(["message" => "Polylang not detected.", "error" => "Polylang is missing. Activate it to use WPTranslation."], 503);
        }

        // get default language
        $defaultLanguage = pll_default_language();
        $languages = pll_the_languages(['raw' => 1]);

        $page = Arr::get($this->request, "filters.page", 1);

        // get posts link to default language
        $args = [
            'lang' => $defaultLanguage,
            "fields" => "id",
            'tax_query' => [
                [
                    'taxonomy' => 'language',
                    'field' => 'term_id',
                    'terms' => $languages[$defaultLanguage]["id"]
                ]
            ]
        ];

        // selected
        if (Arr::get($this->request, "selected.all", false)) {
            $args["nopaging"] = true;
        } else if (Arr::get($this->request, "selected.current", false)) {
            $args["posts_per_page"] = Arr::get($this->request, "per_page", $this->perPage);
            $args["paged"] = $page;
        } else if (!empty(Arr::get($this->request, "selected.selected", []))) {
            $args["nopaging"] = true;
            $args["post__in"] = Arr::get($this->request, "selected.selected");
        } else {
            $this->response([]);
        }

        // make query
        $query = $this->makeQuery($args, Arr::get($this->request, 'filters', []));

        // create output
        $posts = [];
        $languages = Arr::get($this->request, "languages", []);
        foreach ($query->posts as $post) {

            $postTranslations = pll_get_post_translations($post->ID);
            foreach ($languages as $lang) {
                if (isset($postTranslations[$lang])) {
                    $posts[] = $postTranslations[$lang];
                }
            }
        }

        $this->response($posts);
    }

    public function processTranslatePost()
    {
        global $polylang;

        $this->verifyNonce();

        if (!Detector::polylangIsActive()) {
            $this->response(["message" => "Polylang not detected.", "error" => "Polylang is missing. Activate it to use WPTranslation."], 503);
        }

        $this->validate([
            "id" => ["required", "numeric"],
            "lang" => ["required", "string"],
            "refresh" => ["required", "boolean"],
        ]);

        $this->verifyPremium();

        $translator = Translator::make($polylang);

        $id = Arr::get($this->request, "id", null);
        $lang = Arr::get($this->request, "lang", null);
        $refresh = Arr::get($this->request, "refresh", false);

        if ($id && $lang) {
            $translator->duplicatePost($id, $lang, $refresh);
        }

        $this->emptyResponse();
    }

    public function processBackgroundTranslate()
    {
        global $wpdb;

        $this->verifyNonce();

        $this->validate([
            "filters.page" => ["nullable", "numeric"],
            "filters.per_page" => ["nullable", "numeric"],
            "filters.page" => ["nullable", "numeric"],
            "filters.post_type" => ["nullable", "string"],
            "filters.category" => ["nullable", "numeric"],
            "filters.author" => ["nullable", "numeric"],
            "filters.search" => ["nullable", "string"],
            "filters.sort_order" => ["nullable", "string"],
            "filters.sort_direction" => ["nullable", "string"],
            "selected.all" => ["boolean"],
            "selected.current" => ["boolean"],
            "selected.selected" => ["array"],
        ]);

        $this->verifyPremium();

        if (!Detector::polylangIsActive()) {
            $this->response(["message" => "Polylang not detected.", "error" => "Polylang is missing. Activate it to use WPTranslation."], 503);
        }

        // get default language
        $defaultLanguage = pll_default_language();
        $languages = pll_the_languages(['raw' => 1]);

        $page = Arr::get($this->request, "filters.page", 1);

        // get posts link to default language
        $args = [
            'lang' => $defaultLanguage,
            "fields" => "id=>parent",
            'tax_query' => [
                [
                    'taxonomy' => 'language',
                    'field' => 'term_id',
                    'terms' => $languages[$defaultLanguage]["id"]
                ]
            ]
        ];

        // selected
        if (Arr::get($this->request, "selected.all", false)) {
            $args["nopaging"] = true;
        } else if (Arr::get($this->request, "selected.current", false)) {
            $args["posts_per_page"] = Arr::get($this->request, "per_page", $this->perPage);
            $args["paged"] = $page;
        } else if (!empty(Arr::get($this->request, "selected.selected", []))) {
            $args["nopaging"] = true;
            $args["post__in"] = Arr::get($this->request, "selected.selected");
        } else {
            $this->response([]);
        }

        // make query
        $query = $this->makeQuery($args, Arr::get($this->request, 'filters', []));

        // create output
        $posts = [];
        foreach ($query->posts as $post) {
            $translations = [];

            $postTranslations = array_keys(pll_get_post_translations($post->ID));

            foreach (Arr::get($this->request, "languages", []) as $lang) {
                if ($lang === $defaultLanguage) {
                    continue;
                }

                if (!in_array($lang, $postTranslations) || Arr::get($this->request, "refresh", false)) {
                    $translations[] = $lang;
                }
            }

            $posts[] = [
                "id" => $post->ID,
                "parent_id" => $post->post_parent,
                "translations" => $translations,
            ];
        }

        if (empty($posts)) {
            $this->response();
        }

        // remove with empty translations
        $posts = array_filter($posts, function ($post) {
            return !empty($post["translations"]);
        });

        // key by id
        $posts = array_combine(Arr::pluck($posts, 'id'), $posts);

        if (count($posts) > 0) {
            // make tree
            $new = [];
            foreach ($posts as &$post) {
                if (isset($posts[$post['parent_id']])) {
                    $new[$post['parent_id']][] = $post;
                } else {
                    $new[0][] = $post;
                }
            }
            $tree = $this->createTree($new, $new[0]);

            $tableName = $wpdb->prefix . "wpt_translations";
            $now = Carbon::now();

            foreach ($tree as $post) {
                foreach ($post["translations"] as $language) {
                    $saved = $wpdb->insert($tableName, [
                        "post_id" => $post["id"],
                        "language" => $language,
                        "created_at" => $now->format("Y-m-d H:i:s"),
                    ]);
                    if ($saved !== false) {
                        $id = $wpdb->insert_id;

                        $actionId = Scheduler::single("translate_post", [$id]);

                        if (!empty($actionId)) {
                            $wpdb->update(
                                $tableName,
                                [
                                    "action_id" => $actionId,
                                ],
                                [
                                    "id" => $id
                                ]
                            );
                        }
                    }
                }
            }
        }

        $this->response();
    }

    public function processRewritePost()
    {
        global $polylang;

        $this->verifyNonce();

        if (!Detector::polylangIsActive()) {
            $this->response(["message" => "Polylang not detected.", "error" => "Polylang is missing. Activate it to use WPTranslation."], 503);
        }

        $this->validate([
            "id" => ["required", "numeric"],
        ]);

        $this->verifyPremium();

        $translator = Translator::make($polylang);

        $id = Arr::get($this->request, "id", null);

        if ($id) {
            $translator->rewritePost($id);
        }

        $this->emptyResponse();
    }

    private function makeQuery($args, $filters = [])
    {
        $args["post_status"] = "publish";

        // filters
        $category = Arr::get($filters, "category", null);
        $productCategory = Arr::get($filters, "product_category", null);
        if (!empty($category) || !empty($productCategory)) {
            $args["tax_query"] = [];
            if (!empty($category)) {
                $args["tax_query"][] = [
                    'taxonomy' => 'category',
                    'field' => 'term_id',
                    'terms' => $category,
                ];
            }
            if (!empty($productCategory)) {
                $args["tax_query"][] = [
                    'taxonomy' => 'product_cat',
                    'field' => 'term_id',
                    'terms' => $productCategory,
                ];
            }
        }

        $author = Arr::get($filters, "author", null);
        if (!empty($author)) {
            $args["author"] = $author;
        }
        $postType = Arr::get($filters, "post_type", []);
        if (!empty($postType)) {
            $args["post_type"] = $postType;
        } else {
            $args["post_type"] = ['post', 'page'];
        }

        $search = Arr::get($filters, "search", null);
        if (!empty($search)) {
            $args["s"] = $search;
        }

        // order
        $sortOrder = Arr::get($filters, "sort_order", null);
        $sortDirection = Arr::get($filters, "sort_direction", "desc");
        if (!empty($sortOrder)) {
            switch ($sortOrder) {
                case "title";
                    $args["orderby"] = "title";
                    break;
                case "date";
                    $args["orderby"] = "date";
                    break;
            }
            $args["order"] = $sortDirection;
        }

        return new WP_Query($args);
    }

    private function createTree(&$list, $parent)
    {
        $tree = array();
        foreach ($parent as $k => $l) {
            if (isset($list[$l['id']])) {
                $l['children'] = $this->createTree($list, $list[$l['id']]);
            }
            $tree[] = $l;
        }
        return $tree;
    }
}
