<?php

namespace WPTranslation;

use Exception;
use PLL_Admin_Sync;
use WP_Query;
use WPTranslation\Api\GoogleTranslate;
use WPTranslation\Api\OpenAI;
use WPTranslation\Utils\Language;
use WPTranslation\Utils\License;
use WPTranslation\Utils\Scheduler;
use WPTranslation\Utils\Settings;
use WPTranslation\Vendors\Carbon\Carbon;
use WPTranslation\Vendors\DeepL\Translator as DeepLTranslator;
use WPTranslation\Vendors\Illuminate\Support\Arr;
use WPTranslation\Vendors\Illuminate\Support\Str;

if (class_exists("PLL_Admin_Sync")) {

    class Translator extends PLL_Admin_Sync
    {
        static private $instance = null;

        public function __construct(&$polylang)
        {
            parent::__construct($polylang);

            // translate post content
            add_filter('use_block_editor_for_post', [$this, "live_translate"], 5001, 2);

            // translate post title
            add_filter('plt_translate_post_title', [$this, "translate_post_title"], 10, 3);

            // translate post content
            add_filter('plt_translate_post_content', [$this, "translate_post_content"], 10, 3);

            // translate post excerpt
            add_filter('plt_translate_post_excerpt', [$this, "translate_post_excerpt"], 10, 3);

            // translate post meta
            add_filter('pll_translate_post_meta', [$this, "translate_post_meta"], 10, 5);

            // translate post meta
            add_filter('pll_maybe_translate_term', [$this, "maybe_translate_term"], 10, 3);
            add_filter('plt_translate_term_meta', [$this, "translate_term_meta"], 10, 3);

            // save_post hook
            add_action('save_post', [$this, "save_post"], 10, 3);
            add_action('wp_insert_post', [$this, "save_post"], 10, 3);
        }

        static function make(&$polylang)
        {
            if (self::$instance === null) {
                self::$instance = new self($polylang);
            }

            return self::$instance;
        }

        public function live_translate($is_block_editor, $post)
        {
            if (!empty($post) && isset($GLOBALS['pagenow'], $_GET['from_post'], $_GET['new_lang']) && 'post-new.php' === $GLOBALS['pagenow'] && $this->model->is_translated_post_type($post->post_type)) {
                check_admin_referer('new-post-translation');

                $from_post_id = (int) $_GET['from_post'];
                $parent_post = get_post($from_post_id);
                $parent_post_lang = $this->model->post->get_language($from_post_id);
                $post_lang = $this->model->get_language(sanitize_key($_GET['new_lang']));

                if ($this->canTranslatePost($post_lang->slug)) {

                    // copy post content
                    $this->copyPostContent($parent_post, $post);

                    // translate the post content
                    $this->translatePost($post, $post_lang, $parent_post_lang);
                }
            }

            return $is_block_editor;
        }

        public function save_post($post_id, $post, $update)
        {
            global $wpdb;

            if (!License::isPremium()) {
                return;
            }

            // verify that the live is enabled
            if (!Settings::get("live.enabled", false)) {
                return;
            }

            //verify that is not a revision
            if (wp_is_post_revision($post_id)) {
                return;
            }

            // verify that is not an autosave
            if (defined('DOING_AUTOSAVE') and DOING_AUTOSAVE) {
                return;
            }

            // verify that the post is publish
            if ($post->post_status !== "publish") {
                return;
            }

            // verify that polylang functions exists
            if (!function_exists("pll_default_language") || !function_exists("pll_the_languages") || !function_exists("pll_get_post_language")) {
                return;
            }

            // verify that the content is not empty
            if (empty($post->post_content)) {
                return;
            }

            $postLanguage = pll_get_post_language($post_id);
            $defaultLanguage = pll_default_language();
            $allLanguages = pll_the_languages([
                'hide_if_empty' => 0,
                'raw' => 1
            ]);

            // verify that the post is in the default language
            if ($postLanguage !== $defaultLanguage) {
                return;
            }

            $languages = Settings::get("live.languages", []);
            $refresh = Settings::get("live.refresh", false);
            $tableName = $wpdb->prefix . "wpt_translations";
            $now = Carbon::now();

            // quit if a recent translation already exist
            $recentTranslations = $wpdb->get_results("SELECT id from `" . $tableName . "` WHERE `post_id` = " . $post_id . " AND `created_at` >= '" . $now->clone()->subMinute()->format("Y-m-d H:i:s") . "'", ARRAY_A);
            if (!empty($recentTranslations)) {
                return;
            }

            $postTranslations = array_keys(pll_get_post_translations($post_id));

            // remove default language if exist in the language list
            if (($key = array_search($postLanguage, $languages)) !== false) {
                unset($languages[$key]);
            }

            // remove default language
            // remove not enabled languages
            // remove already created language if refresh is false
            $languages = array_values(array_filter($languages, function ($language) use ($postLanguage, $postTranslations, $allLanguages, $refresh) {
                if ($language === $postLanguage) {
                    return false;
                }

                if (!isset($allLanguages[$language])) {
                    return false;
                }

                if (in_array($language, $postTranslations) && !$refresh) {
                    return false;
                }

                return true;
            }));

            // quit if language is empty
            if (empty($languages)) {
                return;
            }

            $postTranslations = array_keys(pll_get_post_translations($post->ID));

            // store the translations
            foreach ($languages as $language) {
                $saved = $wpdb->insert($tableName, [
                    "post_id" => $post_id,
                    "language" => $language,
                    "created_at" => $now->format("Y-m-d H:i:s"),
                ]);
                if ($saved !== false) {
                    $id = $wpdb->insert_id;

                    $actionId = Scheduler::single("translate_post", [$id]);

                    if (!empty($actionId)) {
                        $wpdb->update(
                            $tableName,
                            [
                                "action_id" => $actionId,
                            ],
                            [
                                "id" => $id
                            ]
                        );
                    }
                }
            }
        }

        public function translate_post_meta($value, $key, $target_lang, $from, $to)
        {
            if ($from === $to) {
                return $value;
            }

            $blacklist = [
                "_thumbnail_id",

                // yoast seo
                "_yoast_wpseo_estimated-reading-time-minutes",

                // rank math
                "rank_math_seo_score",
                'rank_math_twitter_use_facebook',
                "rank_math_contentai_score",
                "rank_math_internal_links_processed",

                // seo press
                "_seopress_redirections_type",
                "_seopress_robots_primary_cat",
                "_seopress_redirections_logged_status",
            ];

            if (is_array($value) || empty($value)) {
                return $value;
            }

            if ($this->canTranslatePost($target_lang)) {

                // detect if yoast post meta
                if (in_array($key, [
                    "_yoast_wpseo_focuskw",
                    "_yoast_wpseo_metadesc",
                    "_yoast_wpseo_title",
                    "_yoast_wpseo_bctitle",
                    "_yoast_wpseo_opengraph-title",
                    "_yoast_wpseo_opengraph-description",
                    "_yoast_wpseo_twitter-title",
                    "_yoast_wpseo_twitter-description",
                ])) {
                    if (Settings::get("translation.yoast.enabled", false)) {
                        $service = Settings::get("service", "google-translate-free");
                        $value = $this->translateContent($service, $value, $target_lang, null, "yoast");
                    }
                }
                // detect if rank math post meta
                else if (in_array($key, [
                    "rank_math_focus_keyword",
                    "rank_math_description",
                    "rank_math_title",
                    "rank_math_facebook_title",
                    "rank_math_facebook_description",
                    "rank_math_twitter_description",
                    "rank_math_twitter_title",
                ])) {
                    if (Settings::get("translation.rank_math.enabled", false)) {
                        $service = Settings::get("service", "google-translate-free");
                        $value = $this->translateContent($service, $value, $target_lang, null, "rank_math");
                    }
                }
                // detect if seo press post meta
                else if (in_array($key, [
                    "_seopress_analysis_target_kw",
                    "_seopress_titles_title",
                    "_seopress_titles_desc",
                    "_seopress_social_twitter_desc",
                    "_seopress_social_twitter_title",
                    "_seopress_social_fb_desc",
                    "_seopress_social_fb_title",
                ])) {
                    if (Settings::get("translation.seopress.enabled", false)) {
                        $service = Settings::get("service", "google-translate-free");
                        $value = $this->translateContent($service, $value, $target_lang, null, "seopress");
                    }
                }
                // other post meta
                else if (Settings::get("translation.post_meta.enabled", false)) {

                    // verify blacklist
                    $blacklist = array_merge($blacklist, Settings::get("translation.post_meta.blacklist", []));
                    foreach ($blacklist as $pattern) {
                        if (Str::is($pattern, $key)) {
                            return $value;
                        }
                    }

                    $service = Settings::get("service", "google-translate-free");
                    $value = $this->translateContent($service, $value, $target_lang, null, "html");
                }
            }

            return $value;
        }

        public function maybe_translate_term($translated_term_id, $from_term_id, $target_lang_slug)
        {
            // translated the term meta doesn't exist,
            if (($translated_term_id === 0 || $translated_term_id === false) && Settings::get("translation.terms.enabled", true)) {

                // get the from term
                $from_term = get_term($from_term_id);

                if (!is_wp_error($from_term) && $from_term !== null) {

                    if (in_array($from_term->taxonomy, ["category", "post_tag"])) {

                        // get the from term language
                        $source_lang_slug = pll_get_term_language($from_term->term_id, 'slug');

                        // translate the term of value
                        $name = apply_filters("plt_translate_term_meta", $from_term->name, $target_lang_slug, $source_lang_slug);

                        // create the new term
                        $new_term = wp_insert_term($name, $from_term->taxonomy, [
                            "slug" => sanitize_title($name . '-' . $target_lang_slug)
                        ]);

                        if (!is_wp_error($new_term)) {

                            // set the new term language
                            pll_set_term_language($new_term['term_id'], $target_lang_slug);

                            // get the from term translations
                            $translations = pll_get_term_translations($from_term->term_id);

                            // add the new translation
                            $translations[$target_lang_slug] = $new_term['term_id'];
                            pll_save_term_translations($translations);

                            return $new_term['term_id'];
                        }
                    }
                }
            }

            return $translated_term_id;
        }

        public function translate_post_title($value, $target_lang_slug, $from_lang_slug)
        {
            if (!empty($value) && $this->canTranslatePost($target_lang_slug)) {
                $target_lang = $this->getTranslateLang($target_lang_slug);
                $from_lang = $this->getTranslateLang($from_lang_slug);

                $service = Settings::get("service", "google-translate-free");

                $value = $this->translateContent($service, $value, $target_lang, $from_lang, "text");
            }

            return $value;
        }

        public function translate_post_content($value, $target_lang, $from_lang)
        {
            if (!empty($value) && $this->canTranslatePost($target_lang)) {
                $target_lang = $this->getTranslateLang($target_lang);
                $from_lang = $this->getTranslateLang($from_lang);

                $service = Settings::get("service", "google-translate-free");

                return $this->translateContent($service, $value, $target_lang, $from_lang, "html");
            }
            return $value;
        }

        public function translate_post_excerpt($value, $target_lang, $from_lang)
        {
            if (!empty($value) && $this->canTranslatePost($target_lang)) {
                $target_lang = $this->getTranslateLang($target_lang);
                $from_lang = $this->getTranslateLang($from_lang);

                $service = Settings::get("service", "google-translate-free");

                return $this->translateContent($service, $value, $target_lang, $from_lang, "text");
            }
            return $value;
        }

        public function translate_term_meta($value, $target_lang_slug, $from_lang_slug)
        {
            if (!empty($value) && $this->canTranslatePost($target_lang_slug)) {
                $target_lang = $this->getTranslateLang($target_lang_slug);
                $from_lang = $this->getTranslateLang($from_lang_slug);

                $service = Settings::get("service", "google-translate-free");

                $value = $this->translateContent($service, $value, $target_lang, $from_lang, "text");
            }

            return $value;
        }

        public function duplicatePost($from_post_id, $new_lang, $refresh = false)
        {
            if (!License::isPremium()) {
                return null;
            }

            // verify that polylang functions exists
            if (!function_exists("pll_get_post_translations") || !function_exists("pll_save_post_translations")) {
                return null;
            }

            $from_post = get_post($from_post_id);
            $from_post_lang = $this->model->post->get_language($from_post_id);
            $lang = $this->model->get_language(sanitize_key($new_lang));
            $isNewPost = false;

            if ($from_post && $lang && $from_post_lang && $from_post_lang->term_id !== $lang->term_id && $this->model->is_translated_post_type($from_post->post_type)) {

                if (!$this->canDuplicatePost($from_post, $lang->slug, $refresh)) {
                    return null;
                }

                if (!$this->canTranslatePost($lang->slug)) {
                    return null;
                }

                if ($refresh && $this->hasTranslation($from_post_id, $lang->slug)) {
                    $post = null;

                    $translations = pll_get_post_translations($from_post_id);

                    if (isset($translations[$lang->slug])) {
                        $post = get_post($translations[$lang->slug]);
                    }

                    if (!$post) {
                        return null;
                    }
                } else {
                    $isNewPost = true;

                    // create default post
                    $post = get_default_post_to_edit($from_post->post_type, true);

                    // associate language
                    pll_set_post_language($post->ID, $lang->slug);
                }

                // copy taxonomies and post metas
                $this->taxonomies->copy($from_post_id, $post->ID, $lang->slug);
                $this->post_metas->copy($from_post_id, $post->ID, $lang->slug);

                // add sticky if parent post is sticky
                if (is_sticky($from_post_id)) {
                    stick_post($post->ID);
                }

                // associate parent post
                $this->associatePostParent($from_post, $post);

                // copy post properties
                $this->copyPostProperties($from_post, $post);

                // copy post content
                $this->copyPostContent($from_post, $post);

                // associate translations
                $this->addPostTranslation($from_post, $post);

                // translate the post content
                $this->translatePost($post, $lang, $from_post_lang);

                // associate post author
                if (empty($post->post_author)) {
                    $post->post_author = $from_post->post_author;
                }

                // set post in draft in enabled
                if (Settings::get("set_post_in_draft", false) && $isNewPost) {
                    $post->post_status = "draft";
                }

                // insert the post
                wp_insert_post($post, true);

                return $post;
            }

            return null;
        }

        public function rewritePost($from_post_id)
        {
            if (!License::isPremium()) {
                return null;
            }

            // verify that polylang functions exists
            if (!function_exists("pll_get_post_translations") || !function_exists("pll_save_post_translations")) {
                return null;
            }

            $from_post = get_post($from_post_id);
            $from_post_lang = $this->model->post->get_language($from_post_id);

            if ($from_post && $from_post_lang && $this->model->is_translated_post_type($from_post->post_type)) {

                if (!$this->canTranslatePost($from_post_lang->slug)) {
                    return null;
                }

                // rewite the post content in several languages
                $languages = Language::getRandomLanguages([$from_post_lang->slug], 2);
                $current_language = $from_post_lang->slug;
                foreach ($languages as $new_language) {
                    $from_post->post_content = apply_filters("plt_translate_post_content", $from_post->post_content, $new_language, $current_language);
                    $current_language = $new_language;
                }

                // rewrite the post content in initial language
                $from_post->post_content = apply_filters("plt_translate_post_content", $from_post->post_content, $from_post_lang->slug, $current_language);

                // insert the post
                wp_insert_post($from_post, true);

                return $from_post;
            }

            return null;
        }

        private function addPostTranslation($from_post, $post)
        {
            $translations = pll_get_post_translations($from_post->ID);
            $lang = pll_get_post_language($post->ID, 'slug');
            $translations[$lang] = $post->ID;
            pll_save_post_translations($translations);
        }

        private function associatePostParent($from_post, &$post)
        {
            $parent_post_id = wp_get_post_parent_id($from_post->ID);
            $lang = pll_get_post_language($post->ID, 'slug');
            if ($parent_post_id && ($parent = $this->model->post->get_translation($parent_post_id, $lang))) {
                $post->post_parent = $parent;
            }
        }

        private function copyPostContent($from_post, &$post)
        {
            $post->post_title = $from_post->post_title;
            $post->post_content = $from_post->post_content;
            $post->post_excerpt = $from_post->post_excerpt;
        }

        private function copyPostProperties($from_post, &$post)
        {
            $properties = ['post_status', 'menu_order', 'comment_status', 'ping_status'];
            foreach ($properties as $property) {
                $post->$property = $from_post->$property;
            }

            // Copy the date only if the synchronization is activated
            if (in_array('post_date', $this->options['sync'])) {
                $post->post_date = $from_post->post_date;
                $post->post_date_gmt = $from_post->post_date_gmt;
            }
        }

        private function hasTranslation($post_id, $lang)
        {
            $lang = $this->model->get_language(sanitize_key($lang));
            $translations = pll_get_post_translations($post_id);
            if (isset($translations[$lang->slug])) {
                $translated_post = get_post($translations[$lang->slug]);

                return $translated_post->post_status !== "auto-draft";
            }

            return false;
        }

        private function translatePost(&$post, $target_lang, $source_lang = null)
        {
            if (!empty($post) && $this->model->is_translated_post_type($post->post_type)) {

                $post->post_title = apply_filters("plt_translate_post_title", $post->post_title, $target_lang->slug, $source_lang->slug);

                $post->post_content = apply_filters("plt_translate_post_content", $post->post_content, $target_lang->slug, $source_lang->slug);

                $post->post_excerpt = apply_filters("plt_translate_post_excerpt", $post->post_excerpt, $target_lang->slug, $source_lang->slug);
            }
        }

        private function translateContent($service, $content, $target, $source = null, $mode = "text")
        {
            switch ($service) {
                case "google-translate-free":
                    $api = new GoogleTranslate;
                    if ($mode === "text") {
                        return $api->translate($content, $target, $source);
                    } else if ($mode === "html") {
                        return $api->translateHtml($content, $target, $source);
                    } else if ($mode === "yoast") {
                        return $this->translateYoastContent($service, $content, $target, $source);
                    } else if ($mode === "rank_math") {
                        return $this->translateRankMathContent($service, $content, $target, $source);
                    } else if ($mode === "seopress") {
                        return $this->translateSeopressContent($service, $content, $target, $source);
                    }
                    break;
                case "deepl":
                    if (!License::isPremium()) {
                        return $content;
                    }
                    try {
                        $api = new DeepLTranslator(Settings::get("api.deepl.key"));
                        if (strtolower($target) === "en") {
                            $target = "EN-US";
                        } else if (strtolower($target) === "pt") {
                            $target = "PT-BR";
                        }
                        if ($mode === "text") {
                            $result = $api->translateText($content, $source, $target);
                            return $result->text;
                        } else if ($mode === "html") {
                            $result = $api->translateText($content, $source, $target, [
                                "tag_handling" => "html"
                            ]);
                            return $result->text;
                        } else if ($mode === "yoast") {
                            return $this->translateYoastContent($service, $content, $target, $source);
                        } else if ($mode === "rank_math") {
                            return $this->translateRankMathContent($service, $content, $target, $source);
                        } else if ($mode === "seopress") {
                            return $this->translateSeopressContent($service, $content, $target, $source);
                        }
                    } catch (Exception $e) {
                        return $content;
                    }
                    break;
                case "openai":
                    if (!License::isPremium()) {
                        return $content;
                    }
                    if ($mode === "yoast") {
                        return $this->translateYoastContent("google-translate-free", $content, $target, $source);
                    } else if ($mode === "rank_math") {
                        return $this->translateRankMathContent("google-translate-free", $content, $target, $source);
                    } else if ($mode === "seopress") {
                        return $this->translateSeopressContent("google-translate-free", $content, $target, $source);
                    }
                    try {
                        $sourceLanguage = $this->getPolylangLanguage($source);
                        $language = $this->getPolylangLanguage($target);

                        if (!empty($language)) {
                            $api = new OpenAI();

                            $prompt = [
                                "You are tasked with translating content from one language to another while preserving its original structure and formatting. Follow these instructions carefully:",
                                "",
                                "1. You will be given content to translate. This content may include HTML tags, CSS styles, and other structural elements. Your task is to translate only the text content while keeping all structural elements intact.",
                                "",
                                "2. Here is the content you need to translate:",
                                "<content_to_translate>",
                                $content,
                                "</content_to_translate>",
                                "",
                                "3. The source language for this translation is:",
                                "<source_language>" . $sourceLanguage . "</source_language>",
                                "",
                                "4. The target language for this translation is:",
                                "<target_language>" . $language . "</target_language>",
                                "",
                                "5. Translation guidelines:",
                                "a. Translate all text content within the <content_to_translate> structure, ensuring that every phrase, sentence, and word is translated, unless explicitly identified as a proper noun, brand name, or technical term.",
                                "b. Do not translate or modify any HTML tags, attributes, comments, or CSS styles.",
                                "c. Do not translate if the source language is the same as the target language.",
                                "d. Preserve all line breaks, indentation, and overall formatting of the original content.",
                                "e. Do not translate proper nouns, brand names, or technical terms unless there is a widely accepted translation in the target language.",
                                "",
                                "6. Ensure that your translation is:",
                                "a. Accurate and faithful to the original meaning",
                                "b. Natural and fluent in the target language",
                                "c. Consistent in terminology and style throughout the text",
                                "",
                                "7. Output your translation inside <translated_content> tags without <content_to_translate>. Make sure to include all original structural elements (HTML, CSS, etc.) exactly as they appear in the original content.",
                                "",
                                "8. Do not generate any markdown formatting in your output.",
                                "",
                                "9. Do not provide any explanations or comments outside of the <translated_content> tags.",
                                "",
                                "Begin your translation now.",
                            ];

                            $prompt = implode("\n", $prompt);

                            $model = 'gpt-4o-mini';
                            $result = $api->chat([
                                'model' => $model,
                                'messages' => [
                                    [
                                        "role" => "user",
                                        "content" => $prompt
                                    ]
                                ]
                            ]);

                            if (empty($result["error"])) {
                                $translatedContent = Arr::get($result, "choices.0.message.content");

                                if (!empty($translatedContent)) {
                                    $translatedContent = Str::of($translatedContent)->match('/<translated_content>(.*)<\/translated_content>/s');
                                    $translatedContent = $translatedContent->whenNotEmpty(function ($string) {
                                        $contentToTranslate = $string->match('/<content_to_translate>(.*)<\/content_to_translate>/s');
                                        if ($contentToTranslate->isEmpty()) {
                                            return $string;
                                        }

                                        return $contentToTranslate;
                                    });

                                    $translatedContent = $translatedContent->trim();
                                    $translatedContent = (string) $translatedContent;

                                    return $translatedContent;
                                }
                            }
                        }
                    } catch (Exception $e) {
                        return $content;
                    }
                    break;
            }

            return $content;
        }

        private function translateYoastContent($service, $content, $target, $source)
        {
            // verify deepl and premium
            if ($service === "deepl" && !License::isPremium()) {
                return $content;
            }

            $values = [];
            $index = 1;

            $parts = preg_split("/(%%\w+%%)/", $content, -1, PREG_SPLIT_DELIM_CAPTURE);
            $parts = array_map(function ($part) use (&$values, &$index) {
                // skip if part is empty
                if (strlen(trim($part)) <= 0) {
                    return $part;
                }

                // detect variable
                if (Str::startsWith($part, "%%") && Str::endsWith($part, "%%")) {
                    return $part;
                }

                $key = "INDEX_" . $index++;

                // count left and right spaces
                preg_match_all('/^(\s)+/i', $part, $leftSpaces);
                preg_match_all('/(\s)+$/i', $part, $rigthSpaces);

                $values[] = [
                    "key" => $key,
                    "left_spaces" => count($leftSpaces[0]) > 0 ? strlen($leftSpaces[0][0]) : 0,
                    "right_spaces" => count($rigthSpaces[0]) > 0 ? strlen($rigthSpaces[0][0]) : 0,
                    "value" => trim($part),
                ];

                return $key;
            }, $parts);
            $content = implode("", $parts);

            $texts = array_map(function ($val) {
                return $val["value"];
            }, $values);

            // translate
            switch ($service) {
                case "google-translate-free":
                    $api = new GoogleTranslate;
                    $texts = $api->translate($texts, $target, $source);
                    break;
                case "deepl":
                    $api = new DeepLTranslator(Settings::get("api.deepl.key"));
                    if (strtolower($target) === "en") {
                        $target = "EN-US";
                    } else if (strtolower($target) === "pt") {
                        $target = "PT-BR";
                    }
                    $texts = array_map(function ($text) use ($api, $source, $target) {
                        try {
                            $result = $api->translateText($text, $source, $target, [
                                "tag_handling" => "html"
                            ]);
                            return $result->text;
                        } catch (Exception $e) {
                            return $text;
                        }
                    }, $texts);

                    break;
            }

            $values = array_map(function ($val, $text) {
                $val["value"] = $text;
                return $val;
            }, $values, $texts);

            // reorder values to avoid miss remplacement errors
            usort($values, function ($a, $b) {
                $aKeyIndex = intval(str_replace("INDEX_", "", $a["key"]));
                $bKeyIndex = intval(str_replace("INDEX_", "", $b["key"]));
                return $aKeyIndex < $bKeyIndex;
            });

            // replace placeholder with translated texts
            foreach ($values as $value) {
                if ($value["key"]) {
                    $search = $value["key"];
                    $replacement = str_repeat(" ", $value["left_spaces"] ?? 0) . $value["value"] . str_repeat(" ", $value["right_spaces"] ?? 0);
                    $content = str_replace($search, $replacement, $content);
                }
            }

            return $content;
        }

        private function translateRankMathContent($service, $content, $target, $source)
        {
            // verify deepl and premium
            if ($service === "deepl" && !License::isPremium()) {
                return $content;
            }

            $values = [];
            $index = 1;

            $parts = preg_split("/(%\w+%)/", $content, -1, PREG_SPLIT_DELIM_CAPTURE);
            $parts = array_map(function ($part) use (&$values, &$index) {
                // skip if part is empty
                if (strlen(trim($part)) <= 0) {
                    return $part;
                }

                // detect variable
                if (Str::startsWith($part, "%") && Str::endsWith($part, "%")) {
                    return $part;
                }

                $key = "INDEX_" . $index++;

                // count left and right spaces
                preg_match_all('/^(\s)+/i', $part, $leftSpaces);
                preg_match_all('/(\s)+$/i', $part, $rigthSpaces);

                $values[] = [
                    "key" => $key,
                    "left_spaces" => count($leftSpaces[0]) > 0 ? strlen($leftSpaces[0][0]) : 0,
                    "right_spaces" => count($rigthSpaces[0]) > 0 ? strlen($rigthSpaces[0][0]) : 0,
                    "value" => trim($part),
                ];

                return $key;
            }, $parts);
            $content = implode("", $parts);

            $texts = array_map(function ($val) {
                return $val["value"];
            }, $values);

            // translate
            switch ($service) {
                case "google-translate-free":
                    $api = new GoogleTranslate;
                    $texts = $api->translate($texts, $target, $source);
                    break;
                case "deepl":
                    $api = new DeepLTranslator(Settings::get("api.deepl.key"));
                    if (strtolower($target) === "en") {
                        $target = "EN-US";
                    } else if (strtolower($target) === "pt") {
                        $target = "PT-BR";
                    }
                    $texts = array_map(function ($text) use ($api, $source, $target) {
                        try {
                            $result = $api->translateText($text, $source, $target, [
                                "tag_handling" => "html"
                            ]);
                            return $result->text;
                        } catch (Exception $e) {
                            return $text;
                        }
                    }, $texts);

                    break;
            }

            $values = array_map(function ($val, $text) {
                $val["value"] = $text;
                return $val;
            }, $values, $texts);

            // reorder values to avoid miss remplacement errors
            usort($values, function ($a, $b) {
                $aKeyIndex = intval(str_replace("INDEX_", "", $a["key"]));
                $bKeyIndex = intval(str_replace("INDEX_", "", $b["key"]));
                return $aKeyIndex < $bKeyIndex;
            });

            // replace placeholder with translated texts
            foreach ($values as $value) {
                if ($value["key"]) {
                    $search = $value["key"];
                    $replacement = str_repeat(" ", $value["left_spaces"] ?? 0) . $value["value"] . str_repeat(" ", $value["right_spaces"] ?? 0);
                    $content = str_replace($search, $replacement, $content);
                }
            }

            return $content;
        }

        private function translateSeopressContent($service, $content, $target, $source)
        {
            // verify deepl and premium
            if ($service === "deepl" && !License::isPremium()) {
                return $content;
            }

            $values = [];
            $index = 1;

            $parts = preg_split("/(%%\w+%%)/", $content, -1, PREG_SPLIT_DELIM_CAPTURE);
            $parts = array_map(function ($part) use (&$values, &$index) {
                // skip if part is empty
                if (strlen(trim($part)) <= 0) {
                    return $part;
                }

                // detect variable
                if (Str::startsWith($part, "%%") && Str::endsWith($part, "%%")) {
                    return $part;
                }

                $key = "INDEX_" . $index++;

                // count left and right spaces
                preg_match_all('/^(\s)+/i', $part, $leftSpaces);
                preg_match_all('/(\s)+$/i', $part, $rigthSpaces);

                $values[] = [
                    "key" => $key,
                    "left_spaces" => count($leftSpaces[0]) > 0 ? strlen($leftSpaces[0][0]) : 0,
                    "right_spaces" => count($rigthSpaces[0]) > 0 ? strlen($rigthSpaces[0][0]) : 0,
                    "value" => trim($part),
                ];

                return $key;
            }, $parts);
            $content = implode("", $parts);

            $texts = array_map(function ($val) {
                return $val["value"];
            }, $values);

            // translate
            switch ($service) {
                case "google-translate-free":
                    $api = new GoogleTranslate;
                    $texts = $api->translate($texts, $target, $source);
                    break;
                case "deepl":
                    $api = new DeepLTranslator(Settings::get("api.deepl.key"));
                    if (strtolower($target) === "en") {
                        $target = "EN-US";
                    } else if (strtolower($target) === "pt") {
                        $target = "PT-BR";
                    }
                    $texts = array_map(function ($text) use ($api, $source, $target) {
                        try {
                            $result = $api->translateText($text, $source, $target, [
                                "tag_handling" => "html"
                            ]);
                            return $result->text;
                        } catch (Exception $e) {
                            return $text;
                        }
                    }, $texts);

                    break;
            }

            $values = array_map(function ($val, $text) {
                $val["value"] = $text;
                return $val;
            }, $values, $texts);

            // reorder values to avoid miss remplacement errors
            usort($values, function ($a, $b) {
                $aKeyIndex = intval(str_replace("INDEX_", "", $a["key"]));
                $bKeyIndex = intval(str_replace("INDEX_", "", $b["key"]));
                return $aKeyIndex < $bKeyIndex;
            });

            // replace placeholder with translated texts
            foreach ($values as $value) {
                if ($value["key"]) {
                    $search = $value["key"];
                    $replacement = str_repeat(" ", $value["left_spaces"] ?? 0) . $value["value"] . str_repeat(" ", $value["right_spaces"] ?? 0);
                    $content = str_replace($search, $replacement, $content);
                }
            }

            return $content;
        }

        private function getTranslateLang($slug)
        {
            if (empty($slug)) {
                return null;
            }

            $service = Settings::get("service", "google-translate-free");
            return Language::getLanguageForService($service, $slug);
        }

        private function getPolylangLanguage($slug)
        {
            $languages = pll_the_languages([
                'hide_if_empty' => 0,
                'raw' => 1
            ]);

            if (isset($languages[$slug], $languages[$slug]["name"])) {
                return $languages[$slug]["name"];
            }

            return null;
        }

        private function canDuplicatePost($parent_post, $lang, $refresh = false)
        {
            return $refresh || !$this->hasTranslation($parent_post->ID, $lang);
        }

        private function canTranslatePost($slug)
        {
            return $this->getTranslateLang($slug) !== null;
        }
    }
}
